package org.integrade.asct.ws;

import java.util.Collections;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import asct.shared.ApplicationState;
import asct.shared.IExecutionListener;
import dataTypes.ExecutionRequestId;

public class WSExecutionListener implements IExecutionListener {
	
	private static WSExecutionListener instance;
	private Map<String, List<String>> applicationMap;
	private Map<String, ApplicationState> statusMap;

	public static WSExecutionListener getInstance() {
		if (instance == null)
			instance = new WSExecutionListener();
		return instance;
	}
	
	private WSExecutionListener() {
		statusMap = new HashMap<String, ApplicationState>();
		applicationMap = new HashMap<String, List<String>>();
		
	}
	
	public synchronized void putExecution(String applicationId, String executionId) {
		List<String> execs = applicationMap.get(applicationId);
		if (execs == null) {
			execs = new LinkedList<String>();
			applicationMap.put(applicationId, execs);
		}
		execs.add(executionId);
	}
	
	public synchronized List<String> getExecution(String applicationId){
		List<String> execs = new LinkedList<String>();
		
		if (applicationId.equals("")) {
			for(String key : applicationMap.keySet()) {
				execs.addAll(applicationMap.get(key));
			}
		}
		else
			execs.addAll(applicationMap.get(applicationId));
		return Collections.unmodifiableList(execs);
	}
	
	public synchronized void updateStatus(ExecutionRequestId execRequestId,
			ApplicationState applicationStatus) {
		String id = execRequestId.requestId;
		if (!(statusMap.get(id) == null || statusMap.get(id).equals(ApplicationState.EXECUTING)))
			return;
		
		statusMap.put(id, applicationStatus);

	}

	// returns the application state or null of there is no
	// application execution with the Request ID requestId
	//
	public ApplicationState getStatus(String requestId) {
		ApplicationState status = statusMap.get(requestId);
		return status;
	}
}
