package org.integrade.asct.ws;

import java.io.File;
import java.io.FileOutputStream;

import javax.ws.rs.DELETE;
import javax.ws.rs.POST;
import javax.ws.rs.Path;
import javax.ws.rs.PathParam;
import javax.ws.rs.QueryParam;
import javax.ws.rs.core.Context;
import javax.ws.rs.core.Response;
import javax.ws.rs.core.Response.Status;
import javax.ws.rs.core.UriInfo;

import org.integrade.asct.ws.datatypes.Application;
import org.integrade.asct.ws.datatypes.Binary;

import clusterManagement.ApplicationNotFoundException;

@Path("/application/{application_id:.+}/binaries")
public class WSIntegradeApplicationBinaries {

	private WSApplicationFacade controller = WSApplicationFacade.getInstance();

	@POST
	public Response getIntegradeApplication(@PathParam("application_id") String id, Binary binary,
			@Context UriInfo uriInfo) {

		Response response = null;

		try {
			Application application = controller.getApplicationById(id);

			File f = File.createTempFile("integrade", ".tmp");

			FileOutputStream fout = new FileOutputStream(f);
			fout.write(binary.getFile());
			fout.close();
			
			controller.uploadBinary(f.getAbsolutePath(), application.getBasePath(),
					application.getApplicationName(), binary.getArchitecture());

			response = Response.ok().build();
		} catch (ApplicationNotFoundException anfe) {
			response = Response.status(Status.NOT_FOUND).build();
		} catch (Exception e) {
			e.printStackTrace();
			response = Response.status(Status.INTERNAL_SERVER_ERROR).build();
		}

		return response;
	}

	@DELETE
	public Response unregisterIntegradeApplication(@PathParam("application_id") String id,
			@QueryParam("binaryId") String binaryId, @Context UriInfo uriInfo) {
		Response response = null;

		try {
			Application application = controller.getApplicationById(id);
			controller.deleteBinary(application.getBasePath(), application.getApplicationName(), binaryId);
			response = Response.ok().build();
		} catch (ApplicationNotFoundException anfe) {
			// anfe.printStackTrace();
			response = Response.status(Status.NOT_FOUND).build();
		} catch (Exception e) {
			e.printStackTrace();
			response = Response.status(Status.INTERNAL_SERVER_ERROR).build();
		}

		return response;
	}
}
