package org.integrade.asct.ws;

import java.io.File;
import java.io.FileOutputStream;
import java.util.ArrayList;
import java.util.List;

import javax.ws.rs.GET;
import javax.ws.rs.POST;
import javax.ws.rs.Path;
import javax.ws.rs.QueryParam;
import javax.ws.rs.core.Context;
import javax.ws.rs.core.Response;
import javax.ws.rs.core.UriBuilder;
import javax.ws.rs.core.Response.Status;
import javax.ws.rs.core.UriInfo;

import org.integrade.asct.ws.datatypes.Execution;
import org.integrade.asct.ws.datatypes.ExecutionFile;
import org.integrade.asct.ws.datatypes.Executions;
import org.integrade.asct.ws.datatypes.ParametricCopy;

import clusterManagement.ApplicationNotFoundException;
import dataTypes.ApplicationDescription;
import dataTypes.ExecutionRequestId;
import asct.shared.AbstractGridApplication;
import asct.shared.ApplicationState;
import asct.shared.BspGridApplication;
import asct.shared.ExecutionRequestData;
import asct.shared.ExecutionRequestStatus;
import asct.shared.MpiGridApplication;
import asct.shared.ParametricCopyHolder;
import asct.shared.ParametricGridApplication;
import asct.shared.SequencialGridApplication;

@Path("/executions")
public class WSIntegradeExecutions {

	private WSApplicationFacade controller = WSApplicationFacade.getInstance();

	@GET
	public Response getIntegradeExecution(@QueryParam("applicationId") String appId,  @Context UriInfo uriInfo) {

		Response response = null;

		try {
			if (appId == null)
				appId = "";

			List<String> execIds = WSExecutionListener.getInstance().getExecution(appId);
			Executions executions = new Executions();
			for (String execId : execIds) {
				Execution exec = controller.getExecutionDetails(execId);
				exec.setExecutionId(execId);
				exec.setStatus(WSExecutionListener.getInstance().getStatus(execId).toString());
				executions.getExecution().add(exec);
			}
			
			if (executions.getExecution().isEmpty())
				response = Response.status(Status.NOT_FOUND).build();				
			else		
				response = Response.ok(executions).build();	
		}
		catch (Exception e) {
			e.printStackTrace();
			response = Response.status(Status.INTERNAL_SERVER_ERROR).build();
		}

		return response;
	}
	
	@POST
	public Response createIntegradeExecution (Execution exec, @Context UriInfo uriInfo) {

		Response response = null;
		ExecutionRequestStatus status = null;

		try {
			
			ApplicationDescription appDesc = 
					controller.getApplicationDescriptionById(exec.getApplicationId());
			
			// "dir" is the directory where the input files are stored
			//
			File dir = new File (System.getProperty("integrade.local.dir", 
				     System.getProperty("user.home")) 
				   + File.separator + "input"
				   + File.separator + appDesc.basePath
				   + File.separator + appDesc.applicationName);
		 
			if (! dir.mkdirs()){
				// Error creating upload directory or it already exists
			}
			
			ParametricCopyHolder [] paramCopyHolder = 
					           new ParametricCopyHolder [exec.getNumberOfCopies()];
						
			AbstractGridApplication AGapp;
			
			if (exec.getApplicationType().contains("Parametric")){
				AGapp = new ParametricGridApplication(appDesc.applicationName, 
						appDesc.basePath);
				
				int i = 0;
				for (ParametricCopy pc: exec.getParametricCopies()){
					paramCopyHolder[i] = new ParametricCopyHolder();
					paramCopyHolder[i].setArguments(pc.getArguments());

					// stores the input of each parametric copy
					List<String> inputFileNames = new ArrayList<String>();
					for (ExecutionFile ef: pc.getInputFileNames())
						 inputFileNames.add(dir + File.separator +
								  ef.getFileName());
					paramCopyHolder[i].setInputFiles
					             (inputFileNames.toArray(new String [] {}));
					
					// stores the output of each parametric copy
					List<String> outputFileNames = new ArrayList<String>();
					for (ExecutionFile ef: pc.getOutputFileNames())
						 outputFileNames.add(dir + File.separator +
								 ef.getFileName());
					if (exec.getStoreStdout().contains("True"))
						 outputFileNames.add("stdout");
					if (exec.getStoreStderr().contains("True"))
						 outputFileNames.add("stderr");
					paramCopyHolder[i].setOutputFiles
					             (outputFileNames.toArray(new String [] {}));
					
					paramCopyHolder[i].setCopyId(i+1);
					
					i++;
				}
			}
			
			else if (exec.getApplicationType().contains("BSP")){
				AGapp = new BspGridApplication(appDesc.applicationName, 
						appDesc.basePath);
			}
			else if (exec.getApplicationType().contains("MPI")){
				AGapp = new MpiGridApplication(appDesc.applicationName, 
						appDesc.basePath);
			}
			else {   // Default application: sequential
				AGapp = new SequencialGridApplication(appDesc.applicationName, 
						appDesc.basePath);
			}

			AGapp.setDescription(appDesc);

			ExecutionRequestData data = new ExecutionRequestData ();
									
			data.setApplication(AGapp);
			data.setArguments(exec.getArguments());
			data.setConstraints(exec.getConstraints());
			data.setPreferences(exec.getPreferences());
			
			data.setNumberOfTasks(exec.getNumberOfTasks());
			
			data.setParametricCopies(paramCopyHolder);
			data.setNumberOfCopies(exec.getNumberOfCopies());
			
			if (exec.getForceDifferentNodes().contains("True"))
				data.setForceDifferentMachines(true);
			else
				data.setForceDifferentMachines(false);
			
			/**
			 *   Processing the input files
			 */
			List<String> inputFileNames = new ArrayList<String>();
			
			for (ExecutionFile ef: exec.getInputFiles()){				 
				 File f = new File (dir + File.separator + ef.getFileName());
				 FileOutputStream fout = new FileOutputStream(f);
				 fout.write(ef.getFile());
				 fout.close();
				 inputFileNames.add(f.getAbsolutePath());
			}

			data.setInputFiles(inputFileNames.toArray(new String [] {}));
			
			// Upload InputFiles to OppStore
			String[] inputKeys = controller.uploadFilesToOppStore
					(inputFileNames.toArray(new String [] {}));
			if (inputKeys != null)
				data.setInputFiles(inputKeys);
			
			/**
			 *   Processing the output files
			 */		
			List<String> outputFileNames = new ArrayList<String>();
			
			if (exec.getStoreStdout().contains("True"))
				outputFileNames.add("stdout");

			if (exec.getStoreStderr().contains("True"))
				outputFileNames.add("stderr");
			
			for (ExecutionFile ef: exec.getOutputFiles())
				 outputFileNames.add(ef.getFileName());

			data.setOutputFileNames(outputFileNames.toArray(new String [] {}));
			
			/**
			 * Requests application execution
			 */
			WSExecutionListener listener = WSExecutionListener.getInstance();
			controller.registerExecutionStateListener(listener);
			
			status = controller.executeApplication(data);
			if (status == null){
				System.out.println("Error while executing application!");
			}
			else {
				listener.updateStatus(new ExecutionRequestId(status.getRequestId(), ""), ApplicationState.EXECUTING);
				listener.putExecution(appDesc.appId, status.getRequestId());
			}
			
			UriBuilder uriB = uriInfo.getBaseUriBuilder();
			uriB.path(WSIntegradeExecution.class);
			response = Response.created(uriB.build(status.getRequestId())).build();
		} 
		catch (ApplicationNotFoundException anfe) {
			response = Response.status(Status.NOT_FOUND).build();
		}
		catch (Exception e) {
			e.printStackTrace();
			response = Response.status(Status.INTERNAL_SERVER_ERROR).build();
		}

		return response;	
	}
}
