package org.integrade.asct.ws.datatypes;

import java.util.ArrayList;
import java.util.List;

import javax.xml.bind.annotation.*;


@XmlAccessorType(XmlAccessType.FIELD)
@XmlType (name = "", propOrder = {
	"executionId",
	"applicationId",
	"applicationType",
	"constraints",
	"preferences",
	"arguments",
	"storeStdout",
	"storeStderr",
	"forceDifferentNodes",  // it is the "forceDifferentMachines" field
	"numberOfCopies",
	"numberOfTasks",
	"status",
	"inputFiles",
	"outputFiles",
	"parametricCopies" })
@XmlRootElement(name = "execution")
public class Execution {
	
	private static String NL = System.getProperty("line.separator");

	@XmlElement(required = true)
	private String executionId;
	private String applicationId;
	private String applicationType = "Sequential";
	private String constraints = "";
	private String preferences = "";
	private String arguments = "";
	private String storeStdout = "True";
	private String storeStderr = "True";
	private String forceDifferentNodes = "True";
	private Integer numberOfCopies = 0; // used in parametric executions. 	
	private Integer numberOfTasks = 0; // IMPI: used in BSP executions and MPI executions. 
	private String status;
	@XmlElement( name="executionFile")
    @XmlElementWrapper( name="inputFiles" )
	private List<ExecutionFile> inputFiles = new ArrayList<ExecutionFile>(); 
	@XmlElement( name="executionFile" )
    @XmlElementWrapper( name="outputFiles" )
	private List<ExecutionFile> outputFiles = new ArrayList<ExecutionFile>();
	
	@XmlElement( name="parametricCopy" )
	@XmlElementWrapper( name="parametricCopies" )
	private List<ParametricCopy> parametricCopies = new ArrayList<ParametricCopy>();

	public List<ExecutionFile> getInputFiles() {
		return inputFiles;
	}

	public void setInputFiles(List<ExecutionFile> inputFiles) {
		this.inputFiles = inputFiles;
	}

	public List<ExecutionFile> getOutputFiles() {
		return outputFiles;
	}

	public void setOutputFiles(List<ExecutionFile> outputFiles) {
		this.outputFiles = outputFiles;
	}

	public Execution() {
	}

	public Execution (String appId, String appType, String constraints, 
			          String preferences, String arguments,
	                  String storeStdout, String storeStderr, 
	                  String forceDifferentNodes, 
	                  int numberOfCopies, int numberOfTasks, 
	                  List<ExecutionFile> inputFiles, List<ExecutionFile> outputFiles,
	                  List<ParametricCopy> parametricCopies) {
		this.applicationId = appId;
		this.applicationType = appType;
		this.constraints = constraints;
		this.preferences = preferences;
		this.arguments = arguments;
		this.storeStdout = storeStdout;
		this.storeStderr = storeStderr;
		this.forceDifferentNodes = forceDifferentNodes;
		this.numberOfTasks = numberOfTasks;
		this.inputFiles.addAll(inputFiles);
		this.outputFiles.addAll(outputFiles);
		status = "EXECUTING";		
		this.numberOfCopies = numberOfCopies;
		this.parametricCopies = parametricCopies;
	}

	public String getStatus() {
		return status;
	}

	public void setStatus(String id) {
		this.status = id;
	}
	
	public String getApplicationId() {
		return applicationId;
	}

	public void setApplicationId(String id) {
		this.applicationId = id;
	}

	public String getApplicationType() {
		return applicationType;
	}

	public void setApplicationType(String id) {
		this.applicationType = id;
	}
	
	public String getConstraints() {
		return constraints;
	}

	public void setConstraints(String constraints) {
		this.constraints = constraints;
	}
	
	public String getPreferences() {
		return preferences;
	}

	public void setPreferences(String preferences) {
		this.preferences = preferences;
	}
	
	public String getArguments() {
		return arguments;
	}

	public void setArguments(String arguments) {
		this.arguments = arguments;
	}
	
	public String getStoreStdout() {
		return storeStdout;
	}

	public void setStoreStdout(String storeStdout) {
		this.storeStdout = storeStdout;
	}
	
	public String getStoreStderr() {
		return storeStderr;
	}

	public void setStoreStderr(String storeStderr) {
		this.storeStderr = storeStderr;
	}
	
	public String getForceDifferentNodes() {
		return forceDifferentNodes;
	}

	public void setForceDifferentNodes(String forceDifferentNodes) {
		this.forceDifferentNodes = forceDifferentNodes;	
	}

	public int getNumberOfTasks() {
		return numberOfTasks;
	}

	public void setNumberOfTasks(Integer numberOfTasks) {
		this.numberOfTasks = numberOfTasks;
	}
	
	public int getNumberOfCopies() {
		return numberOfCopies;
	}
	
	public void setNumberOfCopies(Integer numberOfCopies) {
		this.numberOfCopies = numberOfCopies;
	}

		
	public String toString() {
		StringBuffer str = new StringBuffer();
		str.append("applicationId=").append(applicationId).append(NL);
		str.append("applicationType=").append(applicationId).append(NL);
		str.append("constraints=").append(constraints).append(NL);
		str.append("preferences=").append(preferences).append(NL);
		str.append("arguments=").append(arguments).append(NL);
		str.append("storeStdout=").append(storeStdout).append(NL);
		str.append("storeStderr=").append(storeStderr).append(NL);
		str.append("forceDifferentNodes=").append(forceDifferentNodes).append(NL);
		str.append("numberOfTasks=").append(numberOfTasks).append(NL);
		str.append("numberOfCopies=").append(numberOfCopies).append(NL);
		str.append("status=").append(status).append(NL);
		return str.toString();
	}

	public String getExecutionId() {
		return executionId;
	}

	public void setExecutionId(String executionId) {
		this.executionId = executionId;
	}

	public List<ParametricCopy> getParametricCopies() {
		return parametricCopies;
	}

	public void setParametricCopies(List<ParametricCopy> parametricCopies) {
		this.parametricCopies = parametricCopies;
	}
	
}
