package moca.core.proxy.actions.adapters;

import moca.core.proxy.ClientInfo;
import moca.core.proxy.actions.Action;
import moca.core.proxy.message.Message;

/**
 * Classe que especializa a ao para o tipo adapter. Adapter so classes que reagem
 * a um determinado estado de um destinatrio, analisando (e provavelmente modificando) 
 * o contedo de suas mensagens.
 * @author hana
 *
*/
public abstract class Adapter extends Action {
	/**
	 * Faz a adaptao da mensagem de acordo com o estado do destinatrio.
	 * A mensagem resultante da adaptao ser retornada pelo mtodo.  
	 * Caso o adapter resolva interromper o envio da mensagem, ele dever retornar
	 * null.
	 *
	 * @param clientInfo informaes sobre o destinatario
	 * @param msg mensagem a ser adaptada.
	 * @return retorna a mensagem adaptada ou null, caso se deseje que a mensagem no seja enviada
	 * e que o processo de adaptao seja interrompido.
	 * @throws AdaptationException caso algum erro ocorra durante a adaptao
	 * @throws moca.core.proxy.actions.adapters.AdaptationException 
	 *
	 */
	public abstract Message execute(ClientInfo clientInfo, Message msg)
		throws AdaptationException;
}
