/*
 * Created on Dec 5, 2004
 *
 * To change the template for this generated file go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
package moca.core.proxy.actions.listeners;

import moca.core.proxy.ClientInfo;
import moca.core.proxy.receivers.CacheManager;
import moca.core.proxy.receivers.CacheMessage;
import moca.core.proxy.receivers.Dispatcher;
import moca.core.proxy.receivers.Package;

import org.apache.log4j.Logger;

/**
 * This listener implements the default actions to perform cache of 
 * client messages. The cache policy is given by a class tha implements 
 * {@link moca.core.proxy.cache.Cacher}. The class must be specified
 * in the XML configuration file, by the parameter <code>"cacheClassName"</code>.   
 * <p>
 * The method @link #matches(ClientInfo, Dispatcher)
 * is responsible for starting the caching for the specific client, with the
 * cache policy  determined by the <code>cacheClassName</code> parameter. 
 * <p>
 * The method @link #unmatches(ClientInfo, Dispatcher) 
 * creates a control cache message to stop the caching and
 * sends it to the CacheManager. 
 *  
 * @author hana
 *
 */
public class DefaultCacheListener extends StateListener {

	private static final Logger logger = Logger.getLogger(DefaultCacheListener.class);
	
	/* (non-Javadoc)
	 * @see br.edu.puc.proxy.actions.StateListener#matches(br.edu.puc.proxy.ClientInfo, br.edu.puc.proxy.messages.Dispatcher)
	 */
	public void matches(ClientInfo info, Dispatcher disp) {
		logger.info("Cacheando usuario " + info.getClientId());
		String className = (String) this.getParameter("cacheClassName");
		CacheMessage cm = new CacheMessage(className);
		Package pack = new Package(info, cm);
		pack.setDestination(CacheManager.class);
		disp.addPackage(pack);
	}

	/* (non-Javadoc)
	 * @see br.edu.puc.proxy.actions.StateListener#unmatches(br.edu.puc.proxy.ClientInfo, br.edu.puc.proxy.messages.Dispatcher)
	 */
	public void unmatches(ClientInfo info, Dispatcher disp) {
		logger.info("Uncacheando usuario: " + info.getClientId());
		CacheMessage cm = new CacheMessage(CacheMessage.CACHE_STOP);
		Package pack = new Package(info, cm);
		pack.setDestination(CacheManager.class);
		disp.addPackage(pack);
	}
}
