/*
 * Created on 22/10/2004
 *
 * TODO To change the template for this generated file go to
 * Window - Preferences - Java - Code Style - Code Templates
 */
package moca.core.proxy.communication;

import java.io.IOException;
import java.net.InetSocketAddress;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.List;
import java.util.Vector;

import moca.core.proxy.ClientInfo;
import moca.core.proxy.Proxy;
import moca.core.proxy.development.ConnectionListener;
import moca.core.proxy.state.StateConf;
import moca.core.proxy.util.MocaProperties;
import moca.service.cis.client.CisSubscriber;
import moca.protocol.eci.client.listenersmanager.EventListener;
import moca.protocol.net.exception.TimeOutReceiveException;
import moca.service.cis.util.DeviceContext;
import moca.util.eci.ECIConstants;
import moca.util.eci.Event;
import moca.util.eci.Subscription;
import moca.util.eci.Topic;



import org.apache.log4j.Logger;

/**
 *  Responsible for subscribing devices in MoCa CIS service
 *  and handling the events received from it.
 *  
 *  @author hana
 *
 */

public class CISEventProcessing {
	

	private static final Logger logger = Logger.getLogger(CISEventProcessing.class);
	
	private static final String EXPR_CONNECTED = "OnLine = True";
	
	private static boolean ALREADY_EXISTS = false;  // indica que os listeners j foram criados 
		
	// Tag especial -- e por causa dele sempre ter que existir a EXPR_CONNECTED para obter contexto do dispositivo (ter algo para se inscever no CIS)	
	private static final String ALL_DEVICES = "ALL_DEVICES";
	
	/**
	 * CIS subscriber. Responsable to subscribe devices in CIS.
	 */
	private CisSubscriber subscriber;

	/**
	 * List of devices for which subscriptions have been made
	 */
   private List subscriptions;
   
   /**
    * Map expressions to listeners - key: String, value: EventListener  
    */
	private Hashtable exprListeners;
	
	/** List of expressions, that will be used in the subscriptions for CIS*/
	private List expressions;
	
	/** Listener for client connection or disconnection notifications */
	// Por enquanto  s um... precisaria de mais???
	private ConnectionListener clientListener=null;
	
	/**
	 * 
	 * @param proxyAddress
	 * @throws IOException
	 */
	public CISEventProcessing(String proxyAddress)throws IOException {
	   subscriptions = new ArrayList();
	       
		MocaProperties cisProps = new MocaProperties();
				
	   //@TODO: Pegar todos os CIS apartir do CS. Ver uma forma de o CS notificar o LIS quando um CIS foi adicionado ou removido.
	   InetSocketAddress cisPub = new InetSocketAddress(cisProps.getCisServerHost(), cisProps.getCisPublisherPort());
	   InetSocketAddress proxySub = new InetSocketAddress(proxyAddress, 0);
	   subscriber = new CisSubscriber(ECIConstants.TCP, cisPub, proxySub);
	  
	   logger.info("CIS address: " + cisPub);
	   logger.info("Proxy CIS Subscriber: " + proxySub);	   
	      
	   
	}

	/**
	 * Get state expressions from the configurator, to use in the subscriptions to CIS, 
	 * and create the listeners for them.
	 */
	public void configureExpressions(StateConf[] states){

		expressions = new Vector();

		for(int i=0; i< states.length; i++)
		{
			logger.debug("State " + i + ": " + states[i].getExpression().getValue());
			expressions.add(states[i].getExpression().getValue().trim());
		}
		
		createExpressionsListeners();
	}
	
	/**
	 *  Creates listeners for the expressions of interest 
	 *  This information is stored on the Map exprListeners.
	 * 
	 * @param expressions  List of strings representing the expressions of interest
	 */
	private void createExpressionsListeners(){
		
		//TODO: Obs: considera-se que as expresses no mudam,
		// ou seja, que este mtodo  chamado apenas uma vez.
		//Fururamente se formos permitir a alterao da lista de expresses, ento este mtodo
		// dever ser alterado. (--remover listeners, unsubscribe clientes nas antigas expressoes,
 		// subscribe nas novas, etc  OU adicionar as expr novas.. e remover as no mais existentes)
				
		if (ALREADY_EXISTS){
			logger.warn("The expression listeners have already been created");			
		}
		
		ALREADY_EXISTS = true;
		
		exprListeners = new Hashtable();
		 
		//TODO  Pensar se isso no seria a expresso para o cache -- e que deveria ser passada
		// no arquivo de xml.... ex: "OnLine=False AND DeltaT > 10"
		// mas acho que a expesso abaixo tb seria necessria internamente
		
		// A primeira expresso  a de verificar se o dispositivo est ativo
		exprListeners.put(EXPR_CONNECTED, new ConnectedListen());
		
		// Incluir no mapa a expresso e seu repectivo listener
		for(int i=0; i<expressions.size(); i++){
			exprListeners.put((String) expressions.get(i), new ExpressionListen((String) expressions.get(i)));
		}
	}
	
	
	/**
	 * Register the listener for the application proxy be able to receive 
	 * notifications of the connectivity status of clients.
	 * @param listener The listener
	 */
	public void registerConnectionListener(ConnectionListener listener){
		this.clientListener = listener;
	}
	
	/**
	 * Subscribes the device at CIS.  
	 * Makes subscriptions to verify if the device is on-line, and
	 * subscriptions with all expressions configured in the Proxy,
	 *  that were specified in the XML file (that represent the states of interest). 
	 * 
	 * @param  deviceId  - subject for subscribing at CIS
	 */
	
	public void subscribe(String deviceId)  {
		
		int numExpressions = exprListeners.size();
		Enumeration keys = exprListeners.keys();
		String express;
		Topic topic = null;
		
		if (deviceId != null){
			
		    //Ainda nao existe subscricao para esse deviceId
			if (! subscriptions.contains(deviceId)){
				subscriptions.add(deviceId);
				
				for (int i=0; i < numExpressions; i++){	   	  	
					express = (String) keys.nextElement();
					// Se for expresso ALL_DEVICES, no deve cadastrar no CIS 
					//e o estado de contexto sempre ser TRUE
					//TODO: Verificar
					//if (((String)expressions.get(i)).equalsIgnoreCase(ALL_DEVICES) ){ //OU
					
					if(express.equalsIgnoreCase(ALL_DEVICES) ){
//						Montar e enviar evento "fake" indicando que a expresso (estado)  verdadeira...
						Event e = new Event(); 
						e.setExpressionNegation(Event.NORMAL);						
						e.setSubscription(new Subscription(deviceId,new Integer(0)));
						e.setData(null);
						ExpressionListen el = (ExpressionListen)exprListeners.get(express);
						el.onReceiveData(e);  
					}
					else{
						try {
							// Subscrio para ser notificado na primeira vez que o estado ocorre, e 
							// quando deixa de estar vlido
							if(subscriber.getSubscriberStatus(deviceId,express).isSubscribed().booleanValue()){
								subscriber.unsubscribe(new Topic(deviceId,express));
							}
							logger.debug("Subscribing in CIS client " + deviceId + " with expression: " + express);
							topic = subscriber.subscribe(deviceId, express,new Integer(1), new Boolean(true));
							subscriber.addListener((EventListener) exprListeners.get(express), topic);
							
						} catch (TimeOutReceiveException e) {
							// TODO Auto-generated catch block
							logger.warn(e);
						}
					}
				}
			}
		}	    
	}
	

	/**
	 * Unsuscribe in CIS the device and all topics related to it
	 * @param deviceId - the device's identifier
	 */
	public void unsubscribe(String deviceId){
		if (deviceId != null) {
			// Recuperar todas as expresses para remover os topicos do CIS
			for (Enumeration keys = exprListeners.keys(); keys.hasMoreElements() ;) {
				Topic topic = new Topic(deviceId, (String)keys.nextElement());
				subscriber.unsubscribe(topic);
			}
				
			subscriptions.remove(deviceId);				
		}
	}

	/**
	 * Unsuscribe in CIS all devices and all topics related to them
	 */
	public void unsubscribeAll(){

		Iterator it = subscriptions.iterator();
		
		//unsubscribing all devices    
		while(it.hasNext()) {
			String deviceId = (String) it.next();
			
			// Recuperar todas as expresses para remover os topicos do CIS
			for (Enumeration keys = exprListeners.keys(); keys.hasMoreElements() ;) {
				Topic topic = new Topic(deviceId, (String)keys.nextElement());
				subscriber.unsubscribe(topic);
			}
		}
		
		subscriptions.clear();					
	}
		
	/**
	 * Unsubscribe all clients from CIS and remove the listeners. 
	 */
	public void freeResources(){
		unsubscribeAll();
		subscriber.removeAllListeners();
		//TODO: quando tiver  -- chamar o freeResources para o subscriber do CIS
	}
	
	/**
	 * @return Returns the Listeners of the expressions.
	 */
	public Hashtable getExprListeners() {
		return exprListeners;
	}
	/**
	 * @return Returns the subscriber.
	 */
	public CisSubscriber getSubscriber() {
		return subscriber;
	}
	/**
	 * @return Returns the clients that have subscriptions.
	 */
	public List getClients() {
		return subscriptions;
	}
	
	/**
	 * @return Returns the list of expressions (strings).
	 */
	public List getExpressions() {
		return expressions;
	}
	
	/**
	 *  Treat the events of devices connectivity - if it is on-line or not
	 *  
	 */	
	private class ConnectedListen implements EventListener {
		
		// Este listener  apenas para informar que o dispositivo est desconectado, para a 
		// aplicao proxy.... Mas no tem nada a ver com a parte de cache...
		// Para fazer cache, vai ter que ter expresses bem definidas no XML
		// de preferncia 1 para cache de desconexes temporrias e outra para permanente... mas quem define isso
		// o desenvolvedor!!!
		
		/**
		 * Treat the connection event received from CIS.
		 *  It is only to inform the application proxy of this event.
		 */
		public void onReceiveData(Event event) {
			
			// Contexto pesquisado: "OnLine = True"
			
			String device = event.getSubscription().getSubject();
			DeviceContext deviceContext = (DeviceContext) event.getData();
			
			//Atualiza o contexto
			Proxy proxy = Proxy.getInstance();	
			ClientInfo info = proxy.getClientInfo(device);
			info.setCurrentContext(deviceContext);
			
			boolean offline = event.isExpressionNegation().booleanValue();
			String str = (offline)? " OFF" : " ON";
			logger.debug("Received conectivity status of client "+ device + ": " + str );
			logger.debug("Client "+ device + " current context is:  " + deviceContext);
			
			//TODO Avisar a appProxy
			if (clientListener != null){
				//O dispositivo est off-line
				if(offline){					
					clientListener.onReceiveConnectionNotification(device, false);
				}
				// O dispositivo est on-line
				else{					
					clientListener.onReceiveConnectionNotification(device, true);
				}
			}
		}				
				
	}
	
	
	/**
	 * Treat events received from CIS
	 * 
	 * @author hana
	 *
	 */
	private class ExpressionListen implements EventListener {
				
		/**
		 * Expression that identifies the event listener
		 */
	    private String expression;
	    Proxy proxy = Proxy.getInstance();		
				
		public ExpressionListen(String expression){
			this.expression = expression;			
		}
		
		/**
		 * Treat the event received from CIS
		 */
		public void onReceiveData(Event event) {
			
			String device = event.getSubscription().getSubject();
			
			DeviceContext deviceContext = (DeviceContext) event.getData();
			logger.debug("New context of client " + device + ":  " + deviceContext);
						
			// setar se a expresso est ativa (acontecento) OU NO 
			if( event.isExpressionNegation().booleanValue()){
				// TODO checar se o dispositivo ainda est ativo..tvz
				
				logger.debug("State " + expressions.indexOf(expression) + " is FALSE for client " + device);
				// indicar que o estado (expresso) no  vlida para o dispositivo				
				proxy.sendCISMessage(device,expressions.indexOf(expression),false, deviceContext);
			}
			// o estado representado pela expresso est acontecendo
			else{
				logger.debug("State " + expressions.indexOf(expression) + " is TRUE for client " + device);
				proxy.sendCISMessage(device,expressions.indexOf(expression),true, deviceContext);
			}
	    }
		
	  	/**
		 * @return Returns the expression.
		 */
		public String getExpression() {
			return expression;
		}
		/**
		 * @param expression The expression to set.
		 */
		public void setExpression(String expression) {
			this.expression = expression;
		}			
	}

}
