/* ----------------------------------------------------------------------
 * FILE'S INFORMATIONS
 * ----------------------------------------------------------------------
 * Filename.....: TCPSynchronousServer.java
 * Project......: proxy
 * Created on...: 09/10/2004
 * Author.......: hana
 * ----------------------------------------------------------------------
 */

package moca.core.proxy.communication.sync;

import java.io.IOException;

import moca.core.proxy.ClientInfo;
import moca.core.proxy.ClientManager;
import moca.core.proxy.ProxyException;
import moca.core.proxy.communication.ClientCommunication;
import moca.core.proxy.communication.CommunicationProtocol;
import moca.core.proxy.message.Message;
import moca.protocol.net.TCPConnection;
import moca.protocol.net.sync.tcp.SyncTCPServer;

import org.apache.log4j.Category;

/**
 * 
 * @author Hana Rubinsztejn
 */
public class SynchronousTCPServer extends Thread 
{
	private	boolean				isTCPServerRunning = true;
	private	SyncTCPServer 		tcpServer = null;
	private TCPConnection	tcpConnection = null;
	static   Category 			cat = Category.getInstance(SynchronousTCPServer.class.getName());
   
	ClientCommunication clientComm = null;
	
	/**
	 * @param tcpServer
	 * 
	 */
	public SynchronousTCPServer(SyncTCPServer tcpServer, ClientCommunication cc)
	{
		this.tcpServer = tcpServer;
		this.clientComm = cc;
	}

	public void run()
	{
		TCPConnection			tcpConnection = null;
		NewTCPClient			newTCPClient = null;
		
		while(isTCPServerRunning)
		{
			try
			{
				tcpConnection = tcpServer.waitNewConnection();
				
				newTCPClient = new NewTCPClient(tcpConnection);
				newTCPClient.start();		
			}
			catch (IOException e)
			{
				setTCPServerRunning(false);
				cat.warn("Impossible to create one instance of the TCP Server.",e);
				break ;
			}							
		}
	}

	/**
	 * @return
	 */
	public boolean isTCPServerRunning()
	{
		return isTCPServerRunning;
	}

	/**
	 * @param b
	 */
	public void setTCPServerRunning(boolean b)
	{
		isTCPServerRunning = b;
	}

	
	
	/** 
	 *  Handle each message received from TCP protocol
	 * @author hana
	 *
	 */
	private class NewTCPClient extends Thread {
		
		
		private String idClient = null ;
		private ClientManager clientManager = ClientManager.getInstance() ;
		CommunicationProtocol cm = null ;
		
		protected	NewTCPClient(TCPConnection conn)
		{
			tcpConnection = conn;
			
			
			
			// verifica se o cliente ja estava cadastrado e se trata de uma reconexo.
			String idClient = null ;
			if( (idClient = clientManager.getClientId(tcpConnection.getRemoteAddress())) != null){
							
				ClientInfo clientInfo = new ClientInfo(idClient,cm);
				clientManager.register(clientInfo );
			}
		}
		
		/* (non-Javadoc)
		 * @see java.lang.Runnable#run()
		 */
		public void run() {			
			Message msg = null;
			
			// receber mensagens do cliente...
			while(tcpConnection != null && tcpConnection.getSocket().isConnected() && isTCPServerRunning)
			{
				try {

					msg = (Message) tcpConnection.receive();
					
				//	CommunicationProtocol cm = new CommunicationProtocol(CommunicationProtocol.TCP,CommunicationProtocol.SYNC);
				//	cm.setTCPConnection(tcpConnection);
				//	cm.setAddress(tcpConnection.getRemoteAddress() );
					
					cat.debug("Handling TCP message: " + msg);
					
					if (idClient == null)
						idClient = msg.getAddressee() ;
					
					//TODO: PENSAR:  vERIFICAR SE J ERA UM CLIENTE CADASTRADO, E SE
					// J FOR ATUALIZAR ESSA CONEXO-- (PARA O CASO DE DESCONEXO E RECONEXO)			

					try {
						cm = new CommunicationProtocol(CommunicationProtocol.TCP,CommunicationProtocol.SYNC);
						cm.setSyncTCPConnection(tcpConnection);
						cm.setAddress(tcpConnection.getRemoteAddress() );
					} catch (ProxyException e) {
						e.printStackTrace();
					} catch (IOException e) {
						e.printStackTrace();
					}
					//TODO Ser que eu teria que criar outra thread para  chamar o onReceive????r;
					clientComm.notifyReceivedMessage(msg,cm);
					
					//deixar a conexo aberta? e fechar s qdo o usrio for removido...
					// e remover o objeto...e isso tb quando verificar que a conexo no est mais ativo 
					//tcpConnection.close();
				}
				catch (ClassNotFoundException e) {
					cat.error("Error while receiving a synchronous message", e);
					break ;
				}
				catch (IOException e) {
					break ;
			//	} catch (ProxyException e) {
				//	cat.error("Error during CommunicationProtocol initialization" + e);
				//	break ;
				}
			} // while
			try {
				if(tcpConnection != null){
				   tcpConnection.close();
				   tcpConnection.freeResources();
				}
			} catch (Exception e) {
			}
		 
	}
	
	}
	
	public void freeResources(){
		setTCPServerRunning(false);
		try {
			
			if(tcpConnection != null){
				tcpConnection.close();
			    tcpConnection.freeResources();
			}
			if(tcpServer!= null){ 
			    tcpServer.freeResources();    
			}
		} catch (IOException e) {
		}
		tcpServer = null ;
		tcpConnection = null ;
	}
}