package moca.core.proxy.receivers;

import java.util.HashMap;
import java.util.Map;

import moca.core.proxy.ClientInfo;

/**
 * Esta classe  responsvel por intermediar a comunicao entre quaisquer
 * dois receivers. Ela mantem o registro de todos os receivers validos no
 * proxy, associados a um identificador. Assim, de posse do identificador, 
 * ela  capaz de localizar e direcionar um pacote para o receiver correspondente.
 * 
 * @author hana
 */
public class Dispatcher extends Receiver {

	/**
	 * Mapa que relaciona cada um dos receivers e seu identificador
	 */
	private Map destinationMap = new HashMap();

	/**
	 * Adiciona um pacote  fila de processamento do dispatcher
	 * @param pack pacote adicionado
	 */
	public void addPackage(Package pack) {
		super.addPackage(pack);
	}
	/**
	 * Registra um receiver. 
	 * @param key identificador do reciever
	 * @param rec receiver
	 */
	public final void register(Object key, Receiver rec) {
		synchronized (this.destinationMap) {
			this.destinationMap.put(key, rec);
		}
		rec.setDispatcher(this);
	}
	/**
	 * Recupera o receiver de destino
	 * @param key identificador do receiver
	 * @return receiver destino
	 */
	protected final Receiver getDestination(Object key) {
		synchronized (destinationMap) {
			return (Receiver) this.destinationMap.get(key);
		}
	}
	/**
	 * Processa os pacotes adicionados  fila de processamento
	 */
	public final void process() {
		while (true) {
			Package pack = this.getNextPackage();
			Receiver rec = this.getDestination(pack.getDestination());
			rec.addPackage(pack);
		}
	}
	/**
	 * Metodo herdado da classe base. No ser utilizado
	 * @param info informao do destinatario
	 * @param msg mensagem
	 */
	protected ReceiverResult process(ClientInfo info, ReceiverMessage msg) {
		//nao utilizado
		return null;
	}
}
